@@ -1,6 +1,54 @@
-import sys, logging, os
+import re, sys, logging
 from ..html.htmlbuilder import DIV
 
+channel_string = '[USER[:PASSWD]@]HOST[:PORT]|google|file=FILE'
+
+def send(subject, html, channel,
+         send_from=None,
+         send_to=None,
+         send_cc=None,
+         images=None,
+         dry_run=None):
+    """
+    :param subject:     Email subject
+    :param html:        Email content
+    :param channel:     Email delivery channel, or save to file option
+    :param send_from:   Sender's email
+    :param send_to:     Recipients' email(s). Could be array (see below) or string. If string, module pyparsing required
+    :param send_cc:     CC email(s), comment above for send_to applies
+    :param images:      Lost of embedded images
+    :param dry_run:     Dry run (nothing will be sent if True)
+    :return: *Nothing*
+    """
+
+    kwargs = {'send_from' : send_from}
+    if channel == "google" :
+        kwargs['send_with_gmail_api'] = True
+    elif channel.startswith('file=') :
+        kwargs['save_message_to_file'] = channel[5:]
+        kwargs['send_from'] = None  # inhibiting attempt to send
+    else :
+        m = re.compile(r'^(?:(?P<user>[^:]+)(?::(?P<pass>.+))?@)?(?P<host>[a-z0-9.-]+)(?::(?P<port>\d+))?$', re.I).match(channel)
+        if not m :
+            raise ValueError(f"Cannot parse {channel}")
+
+        kwargs['smtp_port'] = m.group('port')
+        kwargs['smtp_host'] = m.group('host')
+        kwargs['smtp_user'] = m.group('user')
+        kwargs['smtp_pass'] = m.group('pass')
+
+    def address(s) :
+        if isinstance(s, str) :
+            return parse_address_string(s)
+        else :
+            return s
+
+    send_email(subject, html,
+               send_to=address(send_to), send_cc=address(send_cc),
+               images=images, dry_run=dry_run,
+               **kwargs)
+
+
 def send_email(subject, html,
                images=None,
                send_to=None,
@@ -113,31 +161,48 @@ def make_list(emails) :
     from email.header import Header
     return ", ".join(("{} <{}>".format(x[0] if x[0].isascii() else Header(x[0], 'utf-8').encode(), x[1]) if x[0] else x[1]) for x in emails)
 
-if __name__ == "__main__" :
+def parse_address_string(s) :
+    import pyparsing as pp
+
+    host = pp.Combine(pp.delimitedList(pp.Word(pp.alphanums), '.', combine=True)).setParseAction(pp.downcaseTokens)
+    email = pp.Combine(pp.Word(pp.alphanums) + "@" + host)
+
+    name = pp.CharsNotIn(',<"') |  pp.dblQuotedString.setParseAction(pp.removeQuotes)
+    recipient = pp.Group(name + "<" + email + ">" | email)
+    rcp_list = pp.delimitedList(recipient, ',') + pp.StringEnd()
+
+    try :
+        return [(None,x[0]) if len(x) == 1 else (x[0], x[2]) for x in rcp_list.parseString(s)]
+    except pp.ParseException as err :
+        logging.error("Cannot parse %s: %s", s, err)
+        return None
+
+def test() :
     import random
-    from colorterm import add_coloring_to_emit_ansi
+    from argparse import ArgumentParser
+    from ..cli.colorterm import add_coloring_to_emit_ansi
+
+    parser = ArgumentParser(description="Testing email send (with embedded image)")
+    parser.add_argument('channel', metavar=channel_string, help="Sending options")
+    parser.add_argument('send_from', metavar='SEND-FROM')
+    parser.add_argument('send_to', metavar='SEND-TO')
+    parser.add_argument('file', metavar='IMAGE')
+
+    args = parser.parse_args ()
 
     logging.basicConfig(format="%(asctime)s.%(msecs)03d %(filename)s:%(lineno)d %(message)s",
                         level=logging.DEBUG, datefmt='%H:%M:%S')
     logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
 
-    if len(sys.argv) != 4 :
-        print(f"Usage: {sys.executable} {sys.argv[0]} <SEND-FROM> <SEND-TO> <FILE>\n"
-              ">This will send samle email to designated address and attach the file<")
-        exit(1)
-
-    addr_from, addr_to, fpath = sys.argv[1:]
-    send_email(subject = f"Проверочный емейл using GMail API, random ID: {random.randrange(10**6, 10**7)}",
-               html = f"""\
+    send(f"Проверочный емейл using , random ID: {random.randrange(10 ** 6, 10 ** 7)}",
+            f"""\
 Hi!<br />
 This is a test of <code>users.messages.send</code>.<br />
-Below we embed image <b>{fpath}</b>, check it out:<br />
+Below we embed image <b>{args.file}</b>, check it out:<br />
 <img src="cid:sample_file" /> <br />
 Hope it worked!
 """,
-               send_from = addr_from,
-               send_to = [("Вася Закусочник", addr_to)],
-               images = {'sample_file': open(fpath,'rb').read()},
-               send_with_gmail_api = True)
-
-
+         args.channel,
+        send_from=args.send_from,
+        send_to=[("Вася Закусочник", args.send_to)],
+        images={'sample_file': open(args.file, 'rb').read()})
